import binascii
from dataclasses import dataclass
from typing import List


@dataclass
class ContentRecord:
    cid: int  # Content ID
    index: int  # Index in the list of contents
    content_type: int  # normal: 0x0001; dlc: 0x4001; shared: 0x8001
    content_size: int
    content_hash: bytearray  # SHA1 hash content


class TMD:
    def __init__(self, tmd):
        self.tmd = tmd
        self.sig_type: int
        self.sig: bytearray
        self.issuer: bytearray  # Root-CA%08x-CP%08x
        self.version: bytearray  # This seems to always be 0 no matter what?
        self.ca_crl_version: int
        self.signer_crl_version: int
        self.vwii: int
        self.ios_tid: str
        self.ios_version: int
        self.title_id: str
        self.title_type: str
        self.group_id: int  # Publisher of the title
        self.region: int
        self.ratings: int
        self.access_rights: int
        self.title_version: int
        self.num_contents: int
        self.boot_index: int
        self.content_record: List[ContentRecord]
        # Load data from TMD file
        with open(tmd, "rb") as tmdfile:
            # Signing certificate issuer
            tmdfile.seek(0x140)
            self.issuer = tmdfile.read(64)
            # TMD version, always seems to be 0?
            tmdfile.seek(0x180)
            self.version = tmdfile.read(1)
            # TODO: label
            tmdfile.seek(0x181)
            self.ca_crl_version = tmdfile.read(1)
            # TODO: label
            tmdfile.seek(0x182)
            self.signer_crl_version = tmdfile.read(1)
            # If this is a vWii title or not
            tmdfile.seek(0x183)
            self.vwii = int.from_bytes(tmdfile.read(1))
            # TID of the IOS to use for the title, set to 0 if this title is the IOS, set to boot2 version if boot2
            tmdfile.seek(0x184)
            ios_version_bin = tmdfile.read(8)
            ios_version_hex = binascii.hexlify(ios_version_bin)
            self.ios_tid = str(ios_version_hex.decode())
            # Get IOS version based on TID
            self.ios_version = int(self.ios_tid[-2:], 16)
            # Title ID of the title
            tmdfile.seek(0x18C)
            title_id_bin = tmdfile.read(8)
            title_id_hex = binascii.hexlify(title_id_bin)
            self.title_id = str(title_id_hex.decode())
            # Type of title
            tmdfile.seek(0x194)
            title_type_bin = tmdfile.read(4)
            title_type_hex = binascii.hexlify(title_type_bin)
            self.title_type = str(title_type_hex.decode())
            # Publisher of the title
            tmdfile.seek(0x198)
            self.group_id = tmdfile.read(2)
            # Region of the title, 0 = JAP, 1 = USA, 2 = EUR, 3 = NONE, 4 = KOR
            tmdfile.seek(0x19C)
            region_hex = tmdfile.read(2)
            self.region = int.from_bytes(region_hex)
            # TODO: figure this one out
            tmdfile.seek(0x19E)
            self.ratings = tmdfile.read(16)
            # Access rights of the title; DVD-video access and AHBPROT
            tmdfile.seek(0x1D8)
            self.access_rights = tmdfile.read(4)
            # Calculate the version number by multiplying 0x1DC by 256 and adding 0x1DD
            tmdfile.seek(0x1DC)
            title_version_high = int.from_bytes(tmdfile.read(1)) * 256
            tmdfile.seek(0x1DD)
            title_version_low = int.from_bytes(tmdfile.read(1))
            self.title_version = title_version_high + title_version_low
            # The number of contents listed in the TMD
            tmdfile.seek(0x1DE)
            self.num_contents = tmdfile.read(2)
            # Content index in content list that contains the boot file
            tmdfile.seek(0x1E0)
            self.boot_index = tmdfile.read(2)

    def get_title_id(self):
        return self.title_id

    def get_title_version(self):
        return self.title_version

    def get_title_region(self):
        match self.region:
            case 0:
                return "JAP"
            case 1:
                return "USA"
            case 2:
                return "EUR"
            case 3:
                return "NONE"
            case 4:
                return "KOR"

    def get_is_vwii_title(self):
        if self.vwii == 1:
            return True
        else:
            return False

    def get_required_ios_tid(self):
        return self.ios_tid

    def get_required_ios(self):
        return self.ios_version

    def get_title_type(self):
        title_id_high = self.title_id[:8]
        match title_id_high:
            case '00000001':
                return "System"
            case '00010000':
                return "Game"
            case '00010001':
                return "Channel"
            case '00010002':
                return "SystemChannel"
            case '00010004':
                return "GameWithChannel"
            case '00010005':
                return "DLC"
            case '00010008':
                return "HiddenChannel"
